import hashlib
from math import sqrt
import heapq
from collections import defaultdict
import svgwrite
import pandas as pd

class Point:
    def __init__(self, x, y):
        self.x = x
        self.y = y

    def distance(self, other):
        return sqrt((self.x - other.x)**2 + (self.y - other.y)**2)

    def __lt__(self, other):
        return (self.x, self.y) < (other.x, other.y)

class Graph:
    def __init__(self):
        self.edges = defaultdict(list)

    def add_edge(self, p1, p2, weight):
        self.edges[p1].append((p2, weight))
        self.edges[p2].append((p1, weight))

    def prim_mst(self):
        mst = []
        visited = set()
        start_node = next(iter(self.edges))
        min_heap = [(0, start_node)]

        while min_heap:
            weight, current = heapq.heappop(min_heap)
            if current not in visited:
                visited.add(current)
                mst.append((current, weight))

                for neighbor, w in self.edges[current]:
                    if neighbor not in visited:
                        heapq.heappush(min_heap, (w, neighbor))

        return mst


# Read dataset
dataset = pd.read_csv('points.csv')


# I - Calculate SHA256 checksum
y_values = "20937235549525372972656649433561976664228369192169403581154174123653112486841409913703020358496193618324279386964310512787403583441458971792584434966088804374104145753441445099413726694947648324342"
hash_object = hashlib.sha256(y_values.encode('utf-8'))
hash_hex = hash_object.hexdigest()
print("SHA256 Checksum: ", hash_hex)

# Instantiate Point objects (if using dataset)
# points = [Point(x, y) for x, y in zip(dataset['x'], dataset['y'])]

# II - Instantiate Point objects (testing data)
x_values = list(range(1, 101))
y_values_str = "20,93,72,35,54,95,25,37,29,72,65,66,49,43,35,61,97,66,64,22,83,69,19,21,69,40,35,81,15,41,74,12,3,65,31,12,48,68,41,40,99,13,70,30,20,35,84,96,1,93,61,83,24,27,93,86,96,43,10,51,27,87,40,35,83,44,15,89,71,79,25,84,43,49,66,0,88,80,4,3,74,10,41,45,75,34,41,44,50,99,41,37,26,6,94,94,76,48,32,42"
y_values = list(map(int, y_values_str.split(',')))
points = [Point(x, y) for x, y in zip(x_values, y_values)]

# III - Find neighbors within specified distance
distance = 20
neighbors = {point: [other for other in points if 0 < point.distance(other) <= distance] for point in points}

# IV - Instantiate Graph object
graph = Graph()
for point, point_neighbors in neighbors.items():
    for neighbor in point_neighbors:
        graph.add_edge(point, neighbor, point.distance(neighbor))

# V - Generate Minimum Spanning Tree
mst = graph.prim_mst()

# VI - Render MST as an SVG image
dwg = svgwrite.Drawing('mst.svg', profile='tiny', viewBox=("0 0 200 200"))
for i in range(1, len(mst)):
    point1, weight1 = mst[i - 1]
    point2, weight2 = mst[i]
    dwg.add(dwg.line((point1.x, point1.y), (point2.x, point2.y), stroke=svgwrite.rgb(0, 0, 0, '%')))
dwg.save()

dwg = svgwrite.Drawing('mst.svg', profile='tiny')
for i in range(1, len(mst)):
    point1, weight1 = mst[i - 1]
    point2, weight2 = mst[i]
    dwg.add(dwg.line((point1.x, point1.y), (point2.x, point2.y), stroke=svgwrite.rgb(0, 0, 0, '%')))
    dwg.add(dwg.circle(center=(point1.x, point1.y), r=2, fill='red'))
    dwg.add(dwg.circle(center=(point2.x, point2.y), r=2, fill='red'))

dwg.save()